﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using VSMSearchSample.ViewModels;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.Search;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Application template is documented at http://go.microsoft.com/fwlink/?LinkId=234227

namespace VSMSearchSample
{
    /// <summary>
    /// Provides application-specific behavior to supplement the default Application class.
    /// </summary>
    sealed partial class App : Application
    {
        /// <summary>
        /// Initializes the singleton application object.  This is the first line of authored code
        /// executed, and as such is the logical equivalent of main() or WinMain().
        /// </summary>
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;

        }

        private static MainViewModel _viewModel;
        public static MainViewModel ViewModel
        {
            get
            {
                if (_viewModel == null)
                    _viewModel = new MainViewModel();

                return _viewModel;
            }
        }

        /// <summary>
        /// Invoked when the application is launched normally by the end user.  Other entry points
        /// will be used when the application is launched to open a specific file, to display
        /// search results, and so forth.
        /// </summary>
        /// <param name="args">Details about the launch request and process.</param>
        protected override void OnLaunched(LaunchActivatedEventArgs args)
        {
            ConfigureSearchContract();

            Frame rootFrame = Window.Current.Content as Frame;

            // Do not repeat app initialization when the Window already has content,
            // just ensure that the window is active
            if (rootFrame == null)
            {
                // Create a Frame to act as the navigation context and navigate to the first page
                rootFrame = new Frame();

                if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    //TODO: Load state from previously suspended application
                }

                // Place the frame in the current Window
                Window.Current.Content = rootFrame;
            }

            if (rootFrame.Content == null)
            {
                // When the navigation stack isn't restored navigate to the first page,
                // configuring the new page by passing required information as a navigation
                // parameter
                if (!rootFrame.Navigate(typeof(MainPage), args.Arguments))
                {
                    throw new Exception("Failed to create initial page");
                }
            }


            // Ensure the current window is active
            Window.Current.Activate();
        }

        /// <summary>
        /// Invoked when application execution is being suspended.  Application state is saved
        /// without knowing whether the application will be terminated or resumed with the contents
        /// of memory still intact.
        /// </summary>
        /// <param name="sender">The source of the suspend request.</param>
        /// <param name="e">Details about the suspend request.</param>
        private void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            //TODO: Save application state and stop any background activity
            deferral.Complete();
        }

        protected override void OnSearchActivated(SearchActivatedEventArgs args)
        {
            ConfigureSearchContract();

            Frame rootFrame = Window.Current.Content as Frame;

            // Do not repeat app initialization when the Window already has content,
            // just ensure that the window is active
            if (rootFrame == null)
            {
                // Create a Frame to act as the navigation context and navigate to the first page
                rootFrame = new Frame();

                // Place the frame in the current Window
                Window.Current.Content = rootFrame;
            }

            if (!rootFrame.Navigate(typeof(SearchPage), args.QueryText))
            {
                throw new Exception("Failed to create initial page");
            }

            // Ensure the current window is active
            Window.Current.Activate();
        }

        public static SearchPane AppSearchPane { get; set; }

        private void ConfigureSearchContract()
        {
            AppSearchPane = SearchPane.GetForCurrentView();

            // Set initial place holder text
            AppSearchPane.PlaceholderText = "Enter something great!";
            AppSearchPane.SuggestionsRequested += AppSearchPane_SuggestionsRequested;
            AppSearchPane.ResultSuggestionChosen += AppSearchPane_ResultSuggestionChosen;

        }

        void AppSearchPane_ResultSuggestionChosen(SearchPane sender, SearchPaneResultSuggestionChosenEventArgs args)
        {
            if (ViewModel != null)
            {
                var id = int.Parse(args.Tag);

                var session = ViewModel.Sessions.Where(s => s.Id == id).First();
                var frame = Window.Current.Content as Frame;

                frame.Navigate(typeof(SessionsPage), session);

            }
        }

        void AppSearchPane_SuggestionsRequested(SearchPane sender, SearchPaneSuggestionsRequestedEventArgs args)
        {
            if (ViewModel != null)
            {
                var cats = ViewModel.Categories
                                    .Where(c => c.Name.ToLower().Contains(args.QueryText.ToLower()))
                                    .Select(c => c.Name)
                                    .Take(2);

                if (cats.Count() > 0)
                {
                    args.Request.SearchSuggestionCollection.AppendQuerySuggestions(cats);
                }

                args.Request.SearchSuggestionCollection.AppendSearchSeparator("Sessions");

                // define an image source
                var imageUri = new Uri("ms-appx:///Assets/search40.png");
                var imageSource = Windows.Storage.Streams.RandomAccessStreamReference.CreateFromUri(imageUri);

                var sessions = ViewModel.Sessions
                                        .Where(s => s.Name.ToLower().Contains(args.QueryText.ToLower()))
                                        .Take(2);

                if (sessions.Count() > 0)
                {
                    foreach (var session in sessions)
                    {
                        args.Request.SearchSuggestionCollection.AppendResultSuggestion(session.Name,
                            session.Description,
                            session.Id.ToString(),
                            imageSource, "");

                    }
                }

            }
        }
    }
}
