﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using VSMSearchSample.Models;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace VSMSearchSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class SearchPage : Page
    {
        public SearchPage()
        {
            this.InitializeComponent();

            PopulateFilters();
        }

        private void PopulateFilters()
        {
            var filters = new List<string>() {
                            "All",
                            "Categories",
                            "Presenters",
                            "Sessions"};

            cbFilter.ItemsSource = filters;
            cbFilter.SelectedValue = "All";
        }
        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            var searchText = e.Parameter.ToString();

            App.ViewModel.SearchFilter = ViewModels.SearchFilterType.All;
            App.ViewModel.Search(searchText);

            UpdateResults();
        }

        private void UpdateResults()
        {
            itemGridView.ItemsSource = App.ViewModel.SearchResults;
            int cnt = App.ViewModel.SearchResults == null ? 0 : App.ViewModel.SearchResults.Count();

            tbQuery.Text = App.ViewModel.SearchQuery + " (" + cnt.ToString() + ")";

        }
        private void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            Frame.Navigate(typeof(SessionsPage), e.ClickedItem);
        }

        private void GoBack(object sender, RoutedEventArgs e)
        {
            if (this.Frame != null && this.Frame.CanGoBack) this.Frame.GoBack();
        }

        private void cbFilter_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            switch (cbFilter.SelectedValue.ToString())
            {
                case "All":
                    App.ViewModel.SearchFilter = ViewModels.SearchFilterType.All;
                    break;
                case "Categories":
                    App.ViewModel.SearchFilter = ViewModels.SearchFilterType.Categories;
                    break;
                case "Presenters":
                    App.ViewModel.SearchFilter = ViewModels.SearchFilterType.Presenters;
                    break;
                case "Sessions":
                    App.ViewModel.SearchFilter = ViewModels.SearchFilterType.Sessions;
                    break;
            }

            UpdateResults();

        }
    }
}
