﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VSMSearchSample.Models;
using System.Xml.Linq;

namespace VSMSearchSample.Utilities
{
    public class TechEdDataParser
    {
        public bool ParseData()
        {
            bool ret = true;

            var itunes = XNamespace.Get("http://www.itunes.com/dtds/podcast-1.0.dtd");
            var dc = XNamespace.Get("http://purl.org/dc/elements/1.1/");

            var sessions = new List<Session>();
            var presenters = new List<Presenter>();
            var categories = new List<Category>();

            var doc = XDocument.Load("Data/TechEdData.xml");

            var items = doc.Element("rss").Element("channel").Elements("item");

            foreach (var item in items)
            {
                var session = new Session();

                session.Id = sessions.Count + 1;
                session.Name = item.Element("title").Value;
                session.Description = item.Element(itunes + "summary").Value;
                session.Link = new Uri(item.Element("guid").Value, UriKind.Absolute);
                session.Categories = new List<Category>();

                var cats = item.Elements("category");

                foreach (var cat in cats)
                {
                    var words = cat.Value.Split(new char[] {'-'});

                    for (int i = 0; i < words.Length; i++)
                    {
                        if(!String.IsNullOrEmpty(words[i]))   
                            words[i] = words[i].Substring(0, 1).ToUpper() + words[i].Substring(1);
                    }

                    var formattedCat = String.Join(" ", words);

                    var qry = categories.Where(c => c.Name == formattedCat);

                    if (qry.Count() > 0)
                    {
                        var curCat = qry.First();

                        curCat.Sessions.Add(session);
                        session.Categories.Add(curCat);
                    }
                    else
                    {
                        var newCat = new Category()
                        {
                            Id = categories.Count + 1,
                            Name = formattedCat,
                            Sessions = new List<Session>()
                        };

                        newCat.Sessions.Add(session);
                        categories.Add(newCat);
                        session.Categories.Add(newCat);
                    }
                }

                session.Presenters = new List<Presenter>();

                var pres = item.Element(dc + "creator").Value.Split(',');

                foreach (var pre in pres)
                {
                    var cur = pre.Trim();

                    if (!String.IsNullOrEmpty(cur))
                    {
                        var qry = presenters.Where(p => p.Name == cur);

                        if (qry.Count() > 0)
                        {
                            var curPre = qry.First();

                            curPre.Sessions.Add(session);
                            session.Presenters.Add(curPre);
                        }
                        else
                        {
                            var newPre = new Presenter()
                            {
                                Id = presenters.Count + 1,
                                Name = cur,
                                Sessions = new List<Session>()
                            };

                            newPre.Sessions.Add(session);
                            presenters.Add(newPre);
                            session.Presenters.Add(newPre);
                        }
                    }
                }
                sessions.Add(session);
            }

            Sessions = sessions.OrderBy(s => s.Name).ToList();
            Presenters = presenters.OrderBy(p => p.Name).ToList();
            Categories = categories.OrderBy(c => c.Name).ToList();


            return ret;
        }

        public IEnumerable<Session> Sessions { get; set; }
        public IEnumerable<Presenter> Presenters { get; set; }
        public IEnumerable<Category> Categories { get; set; }
    }
}
