﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VSMSearchSample.Models;
using VSMSearchSample.Utilities;
using Windows.UI.Xaml.Controls;

namespace VSMSearchSample.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged
    {
        public MainViewModel()
        {
            CreateNavigation();
            PopulateData();
        }

        private void CreateNavigation()
        {
            var navs = new List<NavItem>();
            navs.Add(new NavItem() { Id = "category",
                                     Title = "Categories",
                                     ControlTemplate = (ControlTemplate)App.Current.Resources["CategoryXaml"],
                                     NavigatePage = typeof(CategoryPage)});
            navs.Add(new NavItem() { Id = "presenter",
                                     Title = "Presenters",
                                     ControlTemplate = (ControlTemplate)App.Current.Resources["PeopleXaml"],
                                     NavigatePage = typeof(CategoryPage)});
            NavItems = navs;

        }
        private void PopulateData()
        {
            var parser = new TechEdDataParser();
            if(parser.ParseData()) {
                Presenters = parser.Presenters;
                Sessions = parser.Sessions;
                Categories = parser.Categories;
            }
        }

        private IEnumerable<Presenter> _presenters;
        public IEnumerable<Presenter> Presenters
        {
            get { return _presenters; }
            set
            {
                _presenters = value;
                NotifyPropertyChanged("Presenters");
            }
        }

        private IEnumerable<Session> _sessions;
        public IEnumerable<Session> Sessions
        {
            get { return _sessions; }
            set
            {
                _sessions = value;
                NotifyPropertyChanged("Sessions");
            }
        }

        private IEnumerable<Category> _categories;
        public IEnumerable<Category> Categories
        {
            get { return _categories; }
            set
            {
                _categories = value;
                NotifyPropertyChanged("Categories");
            }
        }

        private IEnumerable<NavItem> _navItems;
        public IEnumerable<NavItem> NavItems
        {
            get { return _navItems; }
            set
            {
                _navItems = value;
                NotifyPropertyChanged("NavItems");
            }
        }

        private SearchFilterType _filter;
        public SearchFilterType SearchFilter
        {
            get { return _filter; }
            set
            {
                _filter = value;
                NotifyPropertyChanged("SearchFilter");
                NotifyPropertyChanged("SearchResults");

            }
        }

        private IEnumerable<BaseModel> _searchResults;
        public IEnumerable<BaseModel> SearchResults
        {
            get { return FilterResults(); }
            set
            {
                _searchResults = value;
                NotifyPropertyChanged("SearchResults");
            }
        }

        public string SearchQuery { get; private set; }

        public void Search(string query)
        {
            SearchQuery = query;
            var lowerQuery = query.ToLower();

            var cats = Categories.Where(c => c.Name.ToLower().Contains(lowerQuery)).Select(s => (BaseModel)s);
            var pres = Presenters.Where(p => p.Name.ToLower().Contains(lowerQuery)).Select(s => (BaseModel)s);
            var sess = Sessions.Where(s => s.Name.ToLower().Contains(lowerQuery)).Select(s => (BaseModel)s);

            var results = cats.Union(pres.Union(sess));

            SearchResults = results.OrderBy(r => r.Name);

        }
        private IEnumerable<BaseModel> FilterResults()
        {
            switch (SearchFilter)
            {
                case SearchFilterType.All:
                    return _searchResults;
                case SearchFilterType.Categories:
                    return _searchResults.Where(r => r.GetType() == typeof(Category));
                case SearchFilterType.Presenters:
                    return _searchResults.Where(r => r.GetType() == typeof(Presenter));
                case SearchFilterType.Sessions:
                    return _searchResults.Where(r => r.GetType() == typeof(Session));

            }

            return null;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    public enum SearchFilterType
    {
        All,
        Presenters,
        Categories,
        Sessions
    }
}
